import { Client, GatewayIntentBits, Collection } from 'discord.js'
import fs from 'node:fs'
import path from 'node:path'
import { config } from './config'

type Command = {
  data: any
}

const client = new Client({
  intents: [GatewayIntentBits.Guilds, GatewayIntentBits.GuildMembers, GatewayIntentBits.GuildMessages, GatewayIntentBits.MessageContent]
})


client.commands = new Collection()

const commandsPath = path.join(__dirname, 'commands');
const commandFiles = fs.readdirSync(commandsPath).filter(file => file.endsWith('.ts'));

(async () => {
  for (const file of commandFiles) {
    const filePath = path.join(commandsPath, file);
    const command = await import(filePath) as Command

    if ('data' in command && 'execute' in command) {
      client.commands.set(command.data.name, command);
    } else {
      console.error(`The command at ${filePath} is missing a required "data" or "execute" property.`);
    }
  }
})

const eventsPath = path.join(__dirname, 'events');
const eventFiles = fs.readdirSync(eventsPath).filter(file => file.endsWith('.ts'));

(async () => {
  for (const file of eventFiles) {
    const filePath = path.join(eventsPath, file);
    const event = await import(filePath);
    if (event.once) {
      client.once(event.name, (...args) => event.execute({ client }, ...args));
    } else {
      client.on(event.name, (...args) => event.execute({ client }, ...args));
    }
  }
})

client.login(config.DISCORD_TOKEN);
