import fs from 'node:fs'
import path from 'node:path'
import { REST, Routes } from 'discord.js'
import { config } from './config'

type Command = {
  data: any
}

export default async () => {
  const commands = []

  const commandsPath = path.join(__dirname, 'commands')
  const commandFiles = fs.readdirSync(commandsPath).filter(file => file.endsWith('ts'))

  for (const file of commandFiles) {
    const filePath = path.join(commandsPath, file)
    const command = await import(filePath) as Command

    if ('data' in command && 'execute' in command) {
      commands.push(command.data.toJSON())
    } else {
      console.error(`The command at ${filePath} is missing a required "data" or "execute" property`)
    }
  }

  const rest = new REST().setToken(config.DISCORD_TOKEN)

  try {
    console.log(`Started refreshing ${commands.length} application (/) commands.`);

		// The put method is used to fully refresh all commands in the guild with the current set
    if (config.DISCORD_GUILD_ID) {
      const data: any = await rest.put(
        Routes.applicationGuildCommands(config.DISCORD_CLIENT_ID, config.DISCORD_GUILD_ID),
        { body: commands },
      );

      console.log(`Successfully reloaded ${data.length} guild (/) commands.`);
    } else {

      const data: any = await rest.put(
        Routes.applicationCommands(config.DISCORD_CLIENT_ID),
        { body: commands },
      );

      console.log(`Successfully reloaded ${data.length} global (/) commands.`);
    }

	} catch (error) {
		// And of course, make sure you catch and log any errors!
		console.error(error);
  }
}
